import matplotlib.pyplot as plt
from numpy import array, load, amax, abs, pi, real, imag, c_, reshape, zeros
from numpy.linalg import norm

from samsara import Samsara, BFGS1_product, cdSY_mat
from problems.JWST.util import data_processor, MgenN, Resize
from problems.JWST.rl import RL_gradRL

EPS = 1e-8


def plot_result(xnew_vec, res):
    # load data
    state = load('state.npz')
    f_vec = state['f_vec']
    ngradf_vec = state['ngradf_vec']
    stepsize_vec = state['stepsize_vec']
    it = state['it']

    t = range(it)
    f1 = plt.figure(1)
    plt.subplot(221)
    plt.semilogy(t, f_vec[t])
    plt.title('Function value')
    plt.xlabel('iterations')
    plt.ylabel('f(x)')

    plt.subplot(222)
    plt.semilogy(t, ngradf_vec[t])
    plt.title('gradient norm')
    plt.xlabel('iteration')
    plt.ylabel('log(gradient norm)')

    plt.subplot(223)
    plt.semilogy(t, stepsize_vec[t])
    plt.title('stepsize')
    plt.xlabel('iteration')
    plt.ylabel('||x_{k+1}-x_k||')

    temp = reshape(xnew_vec, (res, -1), order='F')
    xnew_mat = zeros((res, res, 2))
    xnew_mat[:, :, 0] = temp[:, :res]
    xnew_mat[:, :, 1] = temp[:, res:]

    f2 = plt.figure(2)
    plt.subplot(121)
    plt.imshow(xnew_mat[:, :, 0], extent=[0, 1, 0, 1])
    plt.title('real part of recovered wavefront')

    plt.subplot(122)
    plt.imshow(xnew_mat[:, :, 1], extent=[0, 1, 0, 1])
    plt.title('imaginary part of recovered wavefront')

    plt.show()


def driver():
    # Function to minimise
    Objective = RL_gradRL

    res = 128
    noise = False

    o_params = data_processor(res, noise)
    o_params['dim'] = [res, res, o_params['diversity']]

    # initial guess from file
    im = 1j
    temp_mat = MgenN(res, 2*res, 1e-3*2*pi, 0, 20)
    u_mat = temp_mat[:, :res] + im*temp_mat[:, res:]
    u_mat = u_mat/norm(u_mat, ord='fro')*norm(o_params['k'][:, :, 0])
    u_mat = Resize(u_mat, res, res)
    temp_mat = c_[real(u_mat), imag(u_mat)]
    x0_vec = reshape(temp_mat, 2*res*res, order='F')

    # Tolerances
    ngrad_TOL = 2e-14
    step_TOL = 0.0001*ngrad_TOL
    Maxit = 1500

    # Main loop
    it = 0
    xold_vec = x0_vec
    fold, gradfold_vec = Objective(o_params, xold_vec)

    xnew_vec = None
    stepsize = 999.
    fnew = None
    gradfnew_vec = None
    ngradfnew = 999.

    # Samsara

    options = {}
    options['verbose'] = True
    options['alpha_0'] = 5e-12
    options['gamma'] = .5
    options['c'] = .01
    options['beta'] = .9999999
    options['eta1'] = .995
    options['eta2'] = .8
    options['eta3'] = .25
    # options['eta4'] = .025
    options['maxmem'] = 8
    # options['lookback'] = 0
    options['tr'] = 1e+15

    options['ngrad_TOL'] = 1e-6
    options['step_TOL'] = 1e-12
    options['Maxit'] = 1000

    # options['QN_method'] = Broyden1_product
    # options['Step_finder'] = Explicit_TR
    # options['History'] = cdSY_mat
    # options['update_conditions'] = 'Exact'
    options['initial_step'] = 1e+5*options['step_TOL']
    options['QN_method'] = BFGS1_product
    # options['Step_finder'] = Explicit_TR
    options['History'] = cdSY_mat
    options['update_conditions'] = 'Trust Region'
    options['initial_step'] = 1e-7

    s = Samsara(**options)

    while it < Maxit and ngradfnew > ngrad_TOL and stepsize > step_TOL:
        xnew_vec, xold_vec, fold, gradfold_vec, stepsize =\
            s.run(xold_vec, xnew_vec, fold, fnew,
                  gradfold_vec, gradfnew_vec)

        it += 1
        fnew, gradfnew_vec = Objective(o_params, xnew_vec)
        ngradfnew = norm(gradfnew_vec)

    if stepsize <= step_TOL:
        print('Algorithm stagnated:  stepsize tolerance violated.')
    if it >= Maxit:
        print('Algorithm exceeded:  maximum step count violated.')

    xopt_vec = xnew_vec
    fopt = fnew

    s.save()

    print('fopt =', fopt)

    # s.plot(f_opt=fopt)
    plot_result(xnew_vec, res)

    return xopt_vec, fopt


if __name__ == '__main__':
    driver()
